@@ -25,10 +25,13 @@ callbacks when a state is switched to.
 import logging
 import warnings
 from enum import Enum
-from typing import Callable, Union, List
+from typing import Callable, Union, List, Optional
+
+from ..action.configured import ConfiguredAction, ConfiguredActionList, TrackedAction
 
 
 state_logger = logging.getLogger('hein_control.states')
+state_logger.setLevel(logging.INFO)
 
 
 class State:
@@ -47,8 +50,9 @@ class State:
         """
         self.value = value
         self.alias = alias
-        self.pre_state_callbacks = list(callbacks)
-        self.post_state_callbacks = []
+        self.pre_state_callbacks: ConfiguredActionList = ConfiguredActionList(use_unique_action=True)
+        self.register_pre_state_callbacks(*callbacks)
+        self.post_state_callbacks = ConfiguredActionList(use_unique_action=True)
 
     def __str__(self):
         return f'{self.value} ({len(self.pre_state_callbacks) + len(self.post_state_callbacks)} callbacks)'
@@ -107,37 +111,50 @@ class State:
         """
         return self.register_callbacks(*callbacks, pre_state=False)
 
-    def register_callbacks(self, *callbacks: Callable, pre_state: bool = True):
+    def register_callbacks(self,
+                           *callbacks: Callable,
+                           pre_state: bool = True,
+                           parent_id: Optional[Union[bool, str]] = None
+                           ):
         """
         Registers the provided callbacks with the State instance. These callbacks will be executed when a Component
         state manager switches to this state.
 
         :param callbacks: callbacks to register
         :param pre_state: whether the callbacks should be pre or post state transition
+        :param parent_id: optional parent ID override for created ConfiguredAction instance (e.g. associating a callback
+            with a specific state)
         """
         callback_list = self.pre_state_callbacks if pre_state is True else self.post_state_callbacks
         for callback in callbacks:
             if callable(callback) is False:
                 raise TypeError(f'the provided callback is not callable: {callback}')
-            if callback not in callback_list:
+            if callback_list.action_registered_in_list(callback) is False:
                 state_logger.debug(f'registering {"pre" if pre_state else "post"}-state change {callback} to {self}')
+                callback = ConfiguredAction(
+                    callback,
+                    parent_identifier_override=parent_id or callback_list.parent_identifier_override,
+                )
                 callback_list.append(callback)
 
-    def deregister_callbacks(self, *callbacks: Callable):
+    def deregister_callbacks(self, *callbacks: Callable, remove_configurations: bool = False):
         """
         De-registers callbacks from the State. If a callback is not registered with the instance that callbck will be
         ignored.
 
         :param callbacks: callbacks to remove from the instance
+        :param remove_configurations: option to remove ConfiguredAction from registry after removal from the list
+            (useful for one-off registered methods that will never be used again)
         """
         # todo catch the event that the same callback is registered both before and after the event
         for callback in callbacks:
-            if callback in self.pre_state_callbacks:
-                state_logger.debug(f'removing {callback} from {self} pre-state-change callbacks')
-                self.pre_state_callbacks.remove(callback)
-            elif callback in self.post_state_callbacks:
-                state_logger.debug(f'removing {callback} from {self} post-state-change callbacks')
-                self.post_state_callbacks.remove(callback)
+            for callback_list in [self.pre_state_callbacks, self.post_state_callbacks]:
+                if callback_list.action_registered_in_list(callback) is True:
+                    ca = callback_list.get_ca_by_action(callback)
+                    state_logger.debug(f'removing {ca.name} from {self} callbacks')
+                    callback_list.remove(ca)
+                    if remove_configurations is True:
+                        ConfiguredAction.remove_configuration_from_registry(ca)
 
     def execute_callbacks(self) -> List:
         """executes any registered callbacks and returns their return values in order"""
@@ -148,18 +165,22 @@ class State:
         )
         return self.execute_pre_state_callbacks()
 
-    def execute_pre_state_callbacks(self) -> List:
+    def execute_pre_state_callbacks(self) -> List[TrackedAction]:
         """executes any registered pre-state-change callbacks and returns their values in order"""
         out = []
         for callback in self.pre_state_callbacks:
-            out.append(callback())
+            ta = callback.get_tracked_from_config()
+            ta()
+            out.append(ta)
         return out
 
-    def execute_post_state_callbacks(self) -> List:
+    def execute_post_state_callbacks(self) -> List[TrackedAction]:
         """executes any registered post-state-change callbacks and returns their values in order"""
         out = []
         for callback in self.post_state_callbacks:
-            out.append(callback())
+            ta = callback.get_tracked_from_config()
+            ta()
+            out.append(ta)
         return out
 
 
@@ -261,15 +282,17 @@ class StateSet(Enum):
         """
         return self.register_callbacks(*callbacks, pre_state=False)
 
-    def deregister_callbacks(self, *callbacks: Callable):
+    def deregister_callbacks(self, *callbacks: Callable, remove_configurations: bool = False):
         """
         Pass through for de-registering callbacks with the associated State instance.
 
         :param callbacks: callbacks to remove from the instance
+        :param remove_configurations: option to remove ConfiguredAction from registry after removal from the list
+            (useful for one-off registered methods that will never be used again)
         """
         if isinstance(self.value, State) is False:
             raise TypeError(f'the {self.__class__.__name__} value is not a State instance')
-        self.value.deregister_callbacks(*callbacks)
+        self.value.deregister_callbacks(*callbacks, remove_configurations=remove_configurations)
 
     def execute_callbacks(self) -> List:
         """
