@@ -2,6 +2,7 @@ import time
 import unittest
 from hein_control.action import ConfiguredAction
 from hein_control.scheduler import SamplingScheduler, ExecuteOnTimePoints
+from hein_control.mixin import is_jsonable
 
 
 # logging.basicConfig(level=logging.INFO)
@@ -22,11 +23,17 @@ def fast_step():
     sleepy(0.01)
 
 
+def as_dict_serializable(action) -> bool:
+    """verifies that the as_dict representation is serializable"""
+    return is_jsonable(action.as_dict())
+
+
 class TestScheduler(unittest.TestCase):
     def test_timepoint_addition(self):
         """tests timepoint addition to a scheduler instance"""
         sched = SamplingScheduler('slow_step', fast_step)
         sched.insert_time_point(0.05, number_of_points=4)
+        self.assertTrue(as_dict_serializable(sched))
         self.assertEqual(len(sched.time_points), 4)
         # todo
         #   - test relative insertion
@@ -86,6 +93,7 @@ class TestScheduler(unittest.TestCase):
                 f'test incremental point#{ind + 1}',
                 'check increment of manual name'
             )
+        self.assertTrue(as_dict_serializable(sched))
 
     def test_execution(self):
         """tests execution of the scheduler"""
@@ -111,6 +119,7 @@ class TestScheduler(unittest.TestCase):
         self.assertEqual(sched.completed_time_points, sched.total_time_points, 'complete count is accurate')
         self.assertEqual(sched.uncompleted_time_points, 0, 'no uncompleted time points')
         self.assertFalse(sched.busy)
+        self.assertTrue(as_dict_serializable(sched))
 
     def test_triggering(self):
         """tests manual triggering of a scheduler"""
