@@ -5,6 +5,7 @@ from hein_control.states import (ComponentState, OperationalState, ensure_compon
                                  BadComponentState, GenericActionState, TemporaryOperationalCallback,
                                  TemporaryActionCallback)
 from hein_control.states.abc import State, StateSet
+from hein_control.action.configured import ConfiguredAction
 
 
 class CustomActiveState(StateSet):
@@ -230,15 +231,14 @@ class TestStates(unittest.TestCase):
         self.test_state_one.register_post_state_callbacks(melon)
         self.assertEqual(len(self.test_state_one.pre_state_callbacks), 1, "ensure callback was added")
         self.assertEqual(len(self.test_state_one.post_state_callbacks), 1, 'ensure post callback was added')
-        self.assertTrue(banana in self.test_state_one.pre_state_callbacks, 'check that method is registered')
         self.assertRaises(TypeError, self.test_state_one.register_callbacks, 1, 'check that non-callables are rejected')
 
         # test execution
         values = self.test_state_one.execute_pre_state_callbacks()
         self.assertEqual(len(values), 1, 'check that two returns were captured')
-        self.assertEqual(values[0], 1234, 'check that first function value is correct')
+        self.assertEqual(values[0].action_return, 1234, 'check that first function value is correct')
         post_values = self.test_state_one.execute_post_state_callbacks()
-        self.assertEqual(post_values[0], 'asdf', 'check that second function value is correct')
+        self.assertEqual(post_values[0].action_return, 'asdf', 'check that second function value is correct')
 
         # test deregistration
         self.test_state_one.deregister_callbacks(banana)
@@ -312,8 +312,16 @@ class TestComponentCallbacks(unittest.TestCase):
 
     def test_registration(self):
         """ensure actions were registered as expected"""
-        self.assertTrue(self.component.critical_action in self.component.operational_states.BAD.value.pre_state_callbacks)
-        self.assertTrue(self.component.restore_functionality in self.component.operational_states.GOOD.value.pre_state_callbacks)
+        self.assertIsNotNone(
+            self.component.operational_states.BAD.value.pre_state_callbacks.get_ca_by_action(
+                self.component.critical_action
+            )
+        )
+        self.assertTrue(
+            self.component.operational_states.GOOD.value.pre_state_callbacks.get_ca_by_action(
+                self.component.restore_functionality
+            )
+        )
 
     def test_callback_functionality(self):
         """tests callback functionality"""
@@ -321,23 +329,23 @@ class TestComponentCallbacks(unittest.TestCase):
             try:
                 # change the operational state to a bad state; ensure callback was executed
                 value = self.component.update_operational_state(-1)
-                self.assertEqual(value[0], self.component.bad_string, 'ensure return value is captured')
+                self.assertEqual(value[0].action_return, self.component.bad_string, 'ensure return value is captured')
                 self.assertTrue(self.component.real_bad_thing_happened, 'ensure callback executed as expected')
 
                 # change the operational state to a good one; ensure callback was executed
                 value = self.component.update_operational_state(1)
-                self.assertEqual(value[0], self.component.good_string, 'ensure return value is captured')
+                self.assertEqual(value[0].action_return, self.component.good_string, 'ensure return value is captured')
                 self.assertFalse(self.component.real_bad_thing_happened, 'ensure callback executed as expected')
 
                 # test condition check within restor function
                 value = self.component.update_operational_state(1)
-                self.assertEqual(value[0], self.component.no_change, 'ensure conditional checks work')
+                self.assertEqual(value[0].action_return, self.component.no_change, 'ensure conditional checks work')
 
                 # these will fail if the state is updated before the callbacks are executed
                 value = self.component.update_operational_state(-9000)
-                self.assertEqual(value[0], self.component.allowable_jump)
+                self.assertEqual(value[0].action_return, self.component.allowable_jump)
                 value = self.component.update_action_state(1)
-                self.assertEqual(value[0], self.component.allowable_jump)
+                self.assertEqual(value[0].action_return, self.component.allowable_jump)
             finally:
                 self.component.update_operational_state(1)
                 self.component.update_action_state(0)
@@ -415,6 +423,28 @@ class TestComponentCallbacks(unittest.TestCase):
                 self.component.update_operational_state(1)
                 self.component.update_action_state(0)
 
+    def test_callback_deletion(self):
+        """tests deletion functionality of callbacks"""
+        def some_one_off_function():
+            pass
+        with self.lock:
+            self.component.operational_states(1).value.register_callbacks(some_one_off_function)
+            ca = self.component.operational_states(1).value.pre_state_callbacks[-1]
+            self.assertIsNotNone(ConfiguredAction.class_instance_by_id(ca.name), 'check that retrieval works')
+            self.component.operational_states(1).value.deregister_callbacks(
+                some_one_off_function,
+                remove_configurations=True,
+            )
+            self.assertNotIn(
+                ca, self.component.operational_states(1).value.pre_state_callbacks,
+                'ensure method was removed from callback list'
+            )
+            self.assertRaises(  # ensure that the configured action was removed as expected
+                NameError,
+                ConfiguredAction.class_instance_by_id,
+                ca.name,
+            )
+
 
 class SoManyStates(StateSet):
     SUPER_BAD = State(-10)
@@ -433,6 +463,8 @@ class AllTheStates(ComponentState):
 
 
 class TestTemporaryCallback(unittest.TestCase):
+    change_state_wait_return = None
+
     @classmethod
     def setUpClass(cls) -> None:
         cls.component = AllTheStates('so many states')
@@ -446,18 +478,23 @@ class TestTemporaryCallback(unittest.TestCase):
         """performs the appropriate assignment checks depending on the threshold state"""
         for state in self.component.operational_states:
             if state <= threshold:
-                self.assertTrue(state.has_callbacks, f'ensure callback was assigned to {state}')
+                self.assertTrue(state.has_callbacks, f'ensure a callback is assigned to {state}')
                 self.assertTrue(
-                    fn in state.value.pre_state_callbacks,
+                    state.value.pre_state_callbacks.action_registered_in_list(fn),
                     f'ensure correct callback was assigned to {state}'
                 )
             else:
                 self.assertFalse(state.has_callbacks, 'ensure callback was not assigned to states above threshold')
                 self.assertFalse(
-                    fn in state.value.pre_state_callbacks,
+                    state.value.pre_state_callbacks.action_registered_in_list(fn),
                     f'ensure callback was not assigned to {state}'
                 )
 
+    def check_unassigned(self, fn):
+        """performs a check to ensure that the provided function has been unassigned from all component operational states"""
+        for state in self.component.operational_states:
+            self.assertFalse(state.has_callbacks, f'ensure no callbacks are registered to {state}')
+
     def test_default_callback(self):
         """tests setting of the default callback method"""
         with self.lock:
@@ -482,6 +519,7 @@ class TestTemporaryCallback(unittest.TestCase):
             for threshold_state in self.component.operational_states:
                 with TemporaryOperationalCallback(self.component, threshold_state=threshold_state) as tcb:
                     self.check_assigned(tcb.callback_capture, threshold_state)
+                self.check_unassigned(tcb.callback_capture)
 
     def test_custom_callback(self):
         """tests custom callback setting"""
@@ -489,7 +527,15 @@ class TestTemporaryCallback(unittest.TestCase):
             try:
                 with TemporaryOperationalCallback(self.component, callbacks=self.custom_callback) as tcb:
                     self.check_assigned(self.custom_callback)
-                    self.assertRaises(ValueError, self.change_state_wait)
+                    self.change_state_wait()
+                    self.assertIsNotNone(self.change_state_wait_return)
+                    # load specific tracked actions
+                    callback_capture, custom_callback = self.change_state_wait_return
+                    self.assertEqual(callback_capture.status_code, 3, 'ensure callback capture was triggered')
+                    self.assertEqual(custom_callback.status_code, -1, 'ensure error was raised by custom callback')
+                    self.assertIsNotNone(custom_callback.error_details, 'ensure error details were set')
+                    self.assertIsInstance(custom_callback.error_details, ValueError, 'ensure details are error instance')
+
                 self.assertTrue(tcb.callback_encountered)
                 # ensure callback was unassigned after exiting
                 for state in self.component.operational_states:
@@ -504,7 +550,21 @@ class TestTemporaryCallback(unittest.TestCase):
         inside a TCB context
         """
         time.sleep(0.1)
-        self.component.update_operational_state(-1)
+        self.change_state_wait_return = self.component.update_operational_state(-1)
+
+    def check_registered(self, lst, fn):
+        """checks that the provided callable was registered in the provided ConfiguredActionList"""
+        self.assertTrue(
+            lst.action_registered_in_list(fn),
+            f'ensure {fn} is registered in {lst}'
+        )
+
+    def check_not_registered(self, lst, fn):
+        """checks that the provided callable is not registered in the provided ConfiguredActionList"""
+        self.assertFalse(
+            lst.action_registered_in_list(fn),
+            f'ensure {fn} is not registered in {lst}'
+        )
 
     def test_action_callback(self):
         """tests action callback setting"""
@@ -514,16 +574,10 @@ class TestTemporaryCallback(unittest.TestCase):
             for state in self.component.action_states:
                 if state <= threshold_state:
                     self.assertTrue(state.has_callbacks, f'ensure callback was assigned to {state}')
-                    self.assertTrue(
-                        self.custom_callback in state.value.pre_state_callbacks,
-                        f'ensure correct callback was assigned to {state}'
-                    )
+                    self.check_registered(state.value.pre_state_callbacks, self.custom_callback)
                 elif hasattr(state.value, 'callbacks'):
                     self.assertFalse(state.has_callbacks, 'ensure callback was not assigned to states above threshold')
-                    self.assertFalse(
-                        self.custom_callback in state.value.pre_state_callbacks,
-                        f'ensure callback was not assigned to {state}'
-                    )
+                    self.check_not_registered(state.value.pre_state_callbacks, self.custom_callback)
 
     def test_alias_retrieval(self):
         """tests retrieval of a ComponentState by a state alias"""
@@ -531,4 +585,30 @@ class TestTemporaryCallback(unittest.TestCase):
         alias = 'so_so'
         state.set_alias(alias)
         self.assertIs(state, self.component.operational_states(alias), 'test retrieval of StateSet by alias')
-        self.assertIs(state, self.component.operational_states(state.name), 'ensure retrieval by formal name still works')
\ No newline at end of file
+        self.assertIs(state, self.component.operational_states(state.name), 'ensure retrieval by formal name still works')
+
+    def test_action_removal(self):
+        """tests action removal of temporary callbacks"""
+        def some_other_action():
+            pass
+
+        with TemporaryOperationalCallback(self.component,
+                                          callbacks=[some_other_action],
+                                          remove_configurations_on_exit=True
+                                          ) as tcb:
+            capture_action_name = self.component.operational_states(0).value.pre_state_callbacks[0].name
+            other_action_name = self.component.operational_states(0).value.pre_state_callbacks[1].name
+            self.assertIsNotNone(ConfiguredAction.class_instance_by_id(capture_action_name))
+            self.assertIsNotNone(ConfiguredAction.class_instance_by_id(other_action_name))
+
+        # ensure that actions have been fully deregistered
+        self.assertRaises(
+            NameError,
+            ConfiguredAction.class_instance_by_id,
+            capture_action_name
+        )
+        self.assertRaises(
+            NameError,
+            ConfiguredAction.class_instance_by_id,
+            other_action_name,
+        )
